;------------------------------------------------------
;
;  KS0073 and compatibles in serial mode
;
;  BZ exclusively assigned to SCLK.
;  LED6 dual use as SID and one of the green LEDs.
;  that led is only used by mainline, no need to block interrupts,

PORTC equ 0FA41h
SCLK  equ 40h     ; PC.6 aka BZ
SID   equ 20h     ; PC.5 aka LED6

	section .text
	align 1

;------------------------------------------------------
;
;  4-bit hitachi hung from PCF8574
;  F0 is DB7...4, 08 is E, 04 is RS
;  RS must be valid from leading edge of E,
;  but DBx is latched at E trailing edge (? XXX)
;  RS is kept "1" to speed up data byte transfers.
;
	global _i2c_lcd_cmd, _i2c_lcd_data

nybbles_to_A_B
	ted
	tba
	anda # 0F0h
	aslb
	aslb
	aslb
	aslb
	ord  # 0303h     ; button inputs at P1 and P0 always "1"
	rts

_i2c_lcd_cmd

	bsr  nybbles_to_A_B
	ais  #-6
	tsx

	staa 0,x         ; no E nor RS, valid command bits
	staa 2,x
	oraa # 8
	staa 1,x

	stab 4,x
	orab # 8
	stab 3,x

	ldab # 4+3       ; RS and button inputs
	stab 5,x         ; RS ready for data, again.

	ldd  # 6
	bra  doit

_i2c_lcd_data

	bsr  nybbles_to_A_B
	ais  #-6               ; only 4 used
	tsx

	oraa # 4
	staa 1,x
	oraa # 8
	staa 0,x

	orab # 4
	stab 3,x
	orab # 8
	stab 2,x

	ldd  # 4
doit
	pshm d
	xgdx
	lde  _i2c_lcd_address
	jsr  _i2c_wr

	ais  #+8        ; buffer and also the 3rd arg
	rts

;------------------------------------------------------
;
;  Samsumg serial mode
;
	global _ks0073_cmd, _ks0073_data

_ks0073_cmd                ; void ks0073_cmd(byte cmd);

	ldaa # 1Fh             ; 0 RS=0 R/W=0 11111 command write
	bra  send              ; send E:A, 24 bits

_ks0073_data               ; void ks0073_data(byte data);

	ldaa # 5Fh             ; 0 RS=1 R/W=0 11111 data write
send
	xgde                   ; insert control zeroes to cmd/data byte in E
	tba
	andb # 0Fh             ; zeroes LSnybble
	lsra
	lsra
	lsra
	lsra                   ; zeroes MSnybble
	xgde                   ; byte extended to 16 bits

	pshm k
	ldab # 0FFh
	tbek                   ; EXT FF

	ldab PORTC             ; save LED6
	ldx  # 24              ; total number of bits, in E:A
loop
	bclr PORTC, # SCLK     ; clock low
	lsre
	rora                   ; LSbit first from E:A
	bcs  bit_1
	bclr PORTC, # SID
	bra  clock
bit_1
	bset PORTC, # SID
clock
	bset PORTC, # SCLK     ; data valid at rising clock
	aix  #-1
	bne  loop              ; 24 bits in total

	orab # SCLK            ; make sure clock stays high
	stab PORTC             ; LED6 restored
	pulm k
	rts

;------------------------------------------------------
;
;  Hitachi standard
;
; PC.0 ... PC.3 aka LED1...LED4 wired to DB4 ... DB7, 0Fh in PORTC
; PC.5 aka LED6 wired to RS, 20h in PORTC
; PC.6 aka BZ wired to E,    40h in PORTC

	global _lcd_cmd8, _lcd_cmd, _lcd_data

LCD_ENTER	MACRO
	pshm k
	ldab # 0FFh
	tbek
	ted
	ENDM

lcd_nybble_hi_cmd
	clra

lcd_nybble_hi
	lsrd
	lsrd
	lsrd
	lsrd

lcd_nybble_lo
	orp  # 000E0h  ; <<<<<<<<<<<
	ldaa PORTC     ; save led states, must not never contain 40h
	stab PORTC     ; write RS and DBx
	orab # 40h     ; E up
	stab PORTC     ; write it     XXX too fast ? but it works !
	andb # 0BFh    ; E down
	stab PORTC     ; write it
	nop            ; some hold time for data
	staa PORTC     ; restore led states
	andp # 0FF1Fh  ; >>>>>>>>>>>
	rts

_lcd_cmd8
	LCD_ENTER
	bsr  lcd_nybble_hi_cmd
	bra  lcd_exit

_lcd_cmd
	LCD_ENTER
	bsr  lcd_nybble_hi_cmd
	ted
	andb # 0Fh
	bsr  lcd_nybble_lo
	bra  lcd_exit

_lcd_data
	LCD_ENTER
	ldaa # 2                ; RS before shift
	bsr  lcd_nybble_hi
	ted
	andb # 0Fh
	orab # 20h
	bsr  lcd_nybble_lo
lcd_exit
	pulm k

	ldd #  25      ; it is still more than 40 usec
lcd_delay
	subd # 1       ; assume the loop is more than 100nsec
	bne  lcd_delay ; at least 40 usec, including all overhead in caller
	rts

	end
