
#include <stdio.h>

typedef unsigned char  bool;
typedef unsigned char  byte;
typedef unsigned short word;
typedef unsigned long  dword;

#define FALSE (0)
#define TRUE  (1)

#define __ENDOF(x) ((x) + sizeof(x) / sizeof((x)[0]))

#define INTEREST_IN_CFG 3     /* compilation-time */
#include "hamdr_cfg.h"

static struct cfg cfg;

static void
description(struct cfgparm *p, int html)
{
	int i, j, k, n;
	char c, *cp;

	printf(html ? "<h3>%s</h3>\n\t<blockquote>\n" : "%s\n",
		p->name);

	if (p->group != (byte) ~0) {
		printf("\t");
		cp = "";

		if (p->group & (1 << PERSONALITY_TRACKER)) {
			printf("%sTracker", cp);
			cp = ", ";
		}
		if (p->group & (1 << PERSONALITY_KISS)) {
			printf("%sKISS", cp);
			cp = ", ";
		}
		if (p->group & (1 << PERSONALITY_PEATER)) {
			printf("%sDigipeater", cp);
			cp = ", ";
		}
		if (p->group & (1 << PERSONALITY_VOICE)) {
			printf("%sVoice", cp);
			cp = ", ";
		}
		if (p->group & (1 << PERSONALITY_SCANNER)) {
			printf("%sScanner", cp);
			cp = ", ";
		}

		printf(html ? "<br>\n" : "\n");
	}

	switch (p->type) {
	default:
		abort();
		break;

	case CFG_BOOL:
		printf(html ? "\t<ul><li>%s%s</li><li>%s%s</li></ul>\n"
					: "\t\t%s%s\n\t\t%s%s\n",
			"on", (p->defval ? " DEFAULT" : ""),
			"off", (p->defval ? "" : " DEFAULT"));
		break;

	case CFG_TAB:
		printf(html ? "\t<ul>\n" : "");
		for (j = 0; p->optlist[j]; ++j)
			printf(html ? "\t\t<li>%s%s</li>\n" : "\t\t%s%s\n",
				p->optlist[j], (j == p->defval) ? " DEFAULT" : "");

		printf(html ? "\t</ul>\n" : "");
		break;

	case CFG_APRSYM:
		printf(html ? "\t<ul><li>%d characters</li></ul>\n"
					: "\t\t%d characters\n",
			2);
		break;

	case CFG_CALLSIGN:
		if (p->size != 10)
			abort();
	case CFG_STR:
		printf(html ? "\t<ul><li>%d characters</li></ul>\n"
					: "\t\t%d characters\n",
			p->size - 1);
		break;

	case CFG_WORD:
		printf(html ? "\t<ul><li>0...65535, DEFAULT %d</li></ul>\n"
					: "\t\t0...65535, DEFAULT %d\n",
			p->defval);
		break;

	case CFG_BYTE:
		printf(html ? "\t<ul><li>0...255, DEFAULT %d</li></ul>\n"
					: "\t\t0...255, DEFAULT %d\n",
			p->defval);
		break;

	case CFG_FREQ:
		printf(html ? "\t<ul><li>kHz, DEFAULT %d</li></ul>\n"
					: "\t\tkHz, DEFAULT %d\n",
			p->defval);
		break;
	}
	if (p->descr)
		printf("\n\t%s", p->descr);

	printf("\n");

	printf(html ? "\t</blockquote>\n\n" : "\n");
}

main(int argc, char **argv)
{
	struct cfgparm *p;
	char *mode = argv[1];
	char *notice = "generated file, see hamdr_cfg.h and prconfig.c";

	if (argc != 2) {
usage:
		fprintf(stderr, "Usage: prconfig ascii/html/h/s\n");
		exit(2);
	}

	if (!strcmp(mode, "ascii")
	 || !strcmp(mode, "html")) {

		int html = !strcmp(mode, "html");

		printf(html ? "<h2>%s</h2>\n\n" : "%s\n\n", "general parameters");

		for (p = cfg_parameters; p < __ENDOF(cfg_parameters); ++p)
			description(p, html);

		printf("\n[%s]\n", notice);
		exit(0);
	}
	if (!strcmp(mode, "h")) {

		printf("extern byte /* %s */\n", notice);

		for (p = cfg_parameters; p < __ENDOF(cfg_parameters); ++p) {
			if (p->descr)
				printf("\tdescr_%s[],\n", p->name);
			else
				printf("#define descr_%s 0\n", p->name);
		}
		printf("\thelptext[];\n");
		exit(0);
	}
	if (!strcmp(mode, "s")) {

		printf("\tsection .rodata ; %s\n\n", notice);

		for (p = cfg_parameters; p < __ENDOF(cfg_parameters); ++p) {
			unsigned char *cp = p->descr;

			if (cp) {
				printf("\tglobal _descr_%s\n_descr_%s\n\tdc.b ",
					p->name, p->name);

				while (*cp)
					printf("0%02Xh,", *cp++);
				printf("0\n");
			}
		}
		printf("\talign 1\n\n");
		exit(0);
	}
	goto usage;
}

