
	include regs.inc

	section .text
	align 1

	global _SHT1x_sync, _SHT1x_command, _SHT1x_result

; both pins already have 0 in the output latch, from boot

DAT_PORT equ PORTE
DAT_DDR  equ DDRE
DAT_BIT  equ 80h      ; PE7 aka SIZ1 aka i2c SDA. idle "1" 

CLK_PORT equ PORTF
CLK_DDR  equ DDRF
CLK_BIT  equ 1        ; PF0 aka MODCLK. idle "0"


ENTER	MACRO
	pshm k
	ldab # FF
	tbek
	ENDM

EXIT	MACRO
	pulm k
	rts
	ENDM

;--------------------

_SHT1x_sync         ; void SHT1x_sync(void)
	ENTER
	bsr  CLK_low
	bsr  DAT_high
	ldaa # 9        ; 9 onebits to resync
L0
	bsr  CLK
	deca
	bne  L0
	EXIT            ; CLK low, DAT high

;--------------------

_SHT1x_command      ; bool SHT1x_command(byte cmd)
	ENTER
	bsr  CLK_high
	bsr  DAT_low
	bsr  CLK_low

	bsr  CLK_high
	bsr  DAT_high
	bsr  CLK_low

	xgde            ; command in B
	ldaa # 8
L1
	aslb            ; MSbit first
	bsr  DAT
	deca
	bne  L1

	bsr  DAT_high
	clrd
	bsr  DAT_test
	bcs  L2         ; no ack, return FALSE
	incb            ; ack, return TRUE
L2
	bsr  CLK
	EXIT            ; CLK low, DAT high

;--------------------

DAT                 ; clock out bit from carry
	bcc  DAT_0
DAT_1
	bsr  DAT_high
	bra  CLK
TAD                 ; clock out bit from carry, inverted
	bcc  DAT_1
DAT_0
	bsr  DAT_low
CLK
	bsr  CLK_high
	bsr  CLK_low
	rts

	; bsr/rts gives reasonable delays

CLK_low
	bset CLK_DDR, # CLK_BIT
	rts
CLK_high
	bclr CLK_DDR, # CLK_BIT
	bsr  slowlyyyy
slowlyyyy
	rts

DAT_low
	bset DAT_DDR, # DAT_BIT
	rts
DAT_high
	bclr DAT_DDR, # DAT_BIT
	bsr  slowlyyyy
	rts

DAT_test
	subb # 0
	brclr DAT_PORT, # DAT_BIT, was_0
	orp  # 0100h
was_0
	rts

;--------------------

_SHT1x_result       ; bool SHT1x_result(byte buf[3])
	ENTER
	clrd
	bsr  DAT_test
	bcs  L5         ; chip is stuck, FALSE

	xgex            ; struct { word value; byte crc; }
	ldab # 3        ; 3 bytes
L3
	ldaa # 8        ; 8 bits in a byte
L4
	bsr  DAT_test
	rol  2,x
	rolw 0,x
	bsr  CLK        ; next please
	deca
	bne  L4         ; more bits
	cmpb # 2        ; no carry if 2 or 3
	bsr  TAD        ; send ack bit
	bsr  DAT_high   ; keep released
	decb
	bne  L3         ; more bytes
	ldd  # 1        ; TRUE
L5
	EXIT            ; CLK low, DAT high

