/*
 * Copyright (c) 1982, 1986, 1991 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	From:@(#)in.c	7.17 (Berkeley) 4/20/91
 *	@(#)ax.c	0.0 (Skinnarila) 20/10/93
 */

#include "param.h"
#include "systm.h"
#include "malloc.h"
#include "ioctl.h"
#include "stat.h"
#include "mbuf.h"
#include "socket.h"
#include "socketvar.h"
#include "protosw.h"
#include "domain.h"

#include "net/if.h"
#include "net/route.h"
#include "net/af.h"
#include "net/netisr.h"

#include "ax25/ax.h"
#include "ax25/ax25.h"
#include "ax25/ax_pcb.h"
#include "ax25/ax_var.h"

int ax_debug = 0;

int ax_interfaces;		/* number of external AX25 interfaces */

struct axpcb axb;
struct axpcb axbeacons;
struct axstat axstat;
struct ax_ifaddr *ax_ifaddr;

#define s(x) ((x) << 1)

struct ax_addr ax_multicasts[] = {
	{ s('Q'),s('S'),s('T'),s(' '),s(' '),s(' '), 0 }, /* QST */
	{ s('N'),s('O'),s('D'),s('E'),s('S'),s(' '), 0 }, /* NODES */
	{ s('C'),s('Q'),s(' '),s(' '),s(' '),s(' '), 0 }, /* CQ */
	{ s('A'),s('L'),s('L'),s(' '),s(' '),s(' '), 0 }, /* ALL */
	{ s('B'),s('E'),s('A'),s('C'),s('O'),s('N'), 0 }, /* BEACON */
	{ s('M'),s('A'),s('I'),s('L'),s(' '),s(' '), 0 }, /* MAIL */
	{ s('I'),s('D'),s(' '),s(' '),s(' '),s(' '), 0 }, /* ID */
	{ s('O'),s('P'),s('E'),s('N'),s(' '),s(' '), 0 }, /* OPEN */
	{ s('R'),s('M'),s('N'),s('C'),s(' '),s(' '), 0 }, /* RMNC */
};
struct ax_addr ax_beacons[] = {
	{ s('I'),s('D'),s(' '),s(' '),s(' '),s(' '), 0 }, /* ID */
	{ s('M'),s('A'),s('I'),s('L'),s(' '),s(' '), 0 }, /* MAIL */
	{ s('B'),s('E'),s('A'),s('C'),s('O'),s('N'), 0 }, /* BEACON */
	{ s('A'),s('L'),s('L'),s(' '),s(' '),s(' '), 0 }, /* ALL */
};
struct ax_addr ax_broadcastaddr =
	{ s('Q'),s('S'),s('T'),s(' '),s(' '),s(' '), 0 }; /* QST */

#undef s


ax_init()
{
	axintrq.ifq_maxlen = 30;

	axb.axp_next = axb.axp_prev = &axb;
	axbeacons.axp_next = axbeacons.axp_prev = &axbeacons;

	if (ax_debug)
		printf("ia %d addr %d sax %d axpcb %d MLEN %d MHLEN %d\n",
			sizeof(struct ax_ifaddr),
			sizeof(struct ax_addr),
			sizeof(struct sockaddr_ax),
			sizeof(struct axpcb),
			MLEN, MHLEN,
			0);
}

ax_drain()
{
}

/*
 * Return a printable representation
 * of a AX25 address.
 */
char *
ax_sprintf(a)
	register struct ax_addr *a;
{
	static u_char buf[8][ALEN + sizeof("-15")];
	static char which;
	register u_char *cp;
	register i;

	if (++which >= __SIZEOF(buf))
		which = 0;

	cp = buf[which];
	i = 0;
	do {
		if ((*cp = (a->ax_call[i] >> 1)) != ' ')
			++cp;
	} while (++i < ALEN);
	*cp++ = '-';
	i = (a->ax_ssid & SSID) >> 1;
	if (i >= 10) {
		*cp++ = '1';
		i -= 10;
	}
	*cp++ = '0' + i;
	*cp = 0;
	return (buf[which]);
}

/*
 * Are the addresses equal ?
 * 0 with different callsigns,
 * 1 with same callsigns but different ssids,
 * 2 with identical addresses.
 */
inline
ax_addreq(a1, a2)
	register struct ax_addr *a1;
	register struct ax_addr *a2;
{
	register i;

	i = 0;
	do {
		if ((a1->ax_call[i] ^ a2->ax_call[i]) & ~0x01)
			return (0);
	} while (++i < ALEN);

	if ((a1->ax_ssid ^ a2->ax_ssid) & SSID)
		return (1);
	return (2);
}

/*
 * Return 1 if the address is a multicast address.
 * nonzero SSID --> never a multicast.
 */
int
ax_ismulticast(a)
	register struct ax_addr *a;
{
	register i;

	if ((a->ax_ssid & SSID) != 0)
		return (0);
	i = 0;
	do {
		if (ax_addreq(a, &ax_multicasts[i]))
			return (1);
	} while (++i < __SIZEOF(ax_multicasts));
	return (0);
}

/*
 * Return 1 if the address might be a local broadcast address.
 */
ax_isbroadcast(a)
	struct ax_addr *a;
{
	register struct ax_ifaddr *ia;
	u_long t;

	/*
	 * Look through the list of addresses for a match
	 * with a broadcast address.
	 */
	for (ia = ax_ifaddr; ia; ia = ia->aa_next) {
	    if (ia->aa_ifp->if_flags & IFF_BROADCAST)
			if (ax_addreq(a, &ia->aa_broadaddr.sax_addr))
				 return (1);
	}
	if (ax_addreq(a, &ax_broadcastaddr))
		return (1);
	return (0);
}

char *
ax_framename(ctrl)
	u_char ctrl;
{
	if ((ctrl & 1) == 0)
		return (ctrl & PF ? "INFO/PF" : "INFO");

	switch (ctrl) {
	case UI:      return "UI";
	case UI|PF:   return "UI/PF";

	case SABM:    return "SABM";
	case SABM|PF: return "SABM/PF";

	case UA:      return "UA";
	case UA|PF:   return "UA/PF";

	case DISC:    return "DISC";
	case DISC|PF: return "DISC/PF";

	case FRMR:    return "FRMR";
	case FRMR|PF: return "FRMR/PF";

	case DM:      return "DM";
	case DM|PF:   return "DM/PF";
	}
	switch (ctrl & ~(NR)) {
	case RR:     return "RR";
	case RR|PF:  return "RR/PF";

	case RNR:    return "RNR";
	case RNR|PF: return "RNR/PF";

	case REJ:    return "REJ";
	case REJ|PF: return "REJ/PF";
	}
	return (ctrl & PF ? "???/PF" : "???");
}

ax_trace(title, m)
	char *title;
	struct mbuf *m;
{
	struct axhdr *axh = mtod(m, struct axhdr *);
	u_char cmd = (axh->ah_dst.ax_ssid & SSID_C);
	u_char ctrl = axh->ah_ctrl;
	char *type;

	if (cmd == (axh->ah_src.ax_ssid & SSID_C))
		printf("V1 ");

	printf("%s %d %02x %s -> %s ",
		title, m->m_len, ctrl,
		ax_sprintf(&axh->ah_src), ax_sprintf(&axh->ah_dst));

	switch (ctrl & 3) {
	default:
		printf("INFO %s r=%d s=%d ",
			(ctrl & PF) ? (cmd ? "P" : "F") : (cmd ? "C" : "R"),
			(ctrl >> 5), (ctrl >> 1) & 0x7);
		break;
	case (SABM & 3):
		type = "U???";
		switch (ctrl & ~PF) {
		case UI:      type = "UI  "; break;
		case SABM:    type = "SABM"; break;
		case UA:      type = "UA  "; break;
		case DISC:    type = "DISC"; break;
		case FRMR:    type = "FRMR"; break;
		case DM:      type = "DM  "; break;
		}
		printf("%s %s ", type,
			(ctrl & PF) ? (cmd ? "P" : "F") : (cmd ? "C" : "R"));
		break;
	case (RR & 3):
		type = "S???";
		switch (ctrl & ~(NR|PF)) {
		case RR:     type = "RR  "; break;
		case RNR:    type = "RNR "; break;
		case REJ:    type = "REJ "; break;
		}
		printf("%s %s r=%d ", type,
			(ctrl & PF) ? (cmd ? "P" : "F") : (cmd ? "C" : "R"),
			(ctrl >> 5));
		break;
	}
	if (m->m_len > AXMINLEN) {
		printf("pid %02x len %d\n",
			axh->ah_pid, m->m_pkthdr.len - AXMINLEN);
	} else
		printf("\n");
}

struct ax_ifaddr *
ax_ifawithaddr(a)
	struct ax_addr *a;
{
	register struct ax_ifaddr *ia;

	for (ia = ax_ifaddr; ia; ia = ia->aa_next)
		if (ax_addreq(a, &ia->aa_addr.sax_addr) > 1)
			break;

	return (ia);
}

struct ax_ifaddr *
ax_ifawithssid(a)
	struct ax_addr *a;
{
	register struct ax_ifaddr *ia;

	for (ia = ax_ifaddr; ia; ia = ia->aa_next)
		if (a->ax_ssid == ia->aa_addr.sax_addr.ax_ssid)
			break;

	return (ia);
}

struct ax_ifaddr *
ax_ifawithifname(name)
	char *name;
{
	struct ax_ifaddr *ia;
	char *cp;
	int unit = 0;
	int namelen;

	for (cp = name; cp < &name[IFNAMSIZ] && *cp; ++cp)
		;
	while (--cp > name && *cp >= '0' && *cp <= '9')
		;
	namelen = ++cp - name;

	unit = *cp - '0';
	if (*++cp)
			unit = 10 * unit + *cp - '0';

	for (ia = ax_ifaddr; ia; ia = ia->aa_next)
		if (ia->aa_ifp->if_unit == unit
		 && !bcmp(ia->aa_ifp->if_name, name, namelen))
			break;
	return (ia);
}

struct ifnet *
ax_iflookup(name)
	char *name;
{
	struct ax_ifaddr *ifa;

	if (ifa = ax_ifawithifname(name))
		return ifa->aa_ifp;
	return (NULL);
}

/*
 * Generic AX25 control operations (ioctl's).
 * Ifp is 0 if not an interface-specific ioctl.
 */
/* ARGSUSED */
ax_control(so, cmd, data, ifp)
	struct socket *so;
	int cmd;
	caddr_t data;
	register struct ifnet *ifp;
{
	register struct ifreq *ifr = (struct ifreq *)data;
	struct ax_aliasreq *ifra = (struct ax_aliasreq *)data;
	register struct ax_ifaddr *ia = 0;
	register struct ifaddr *ifa;
	struct ax_ifaddr *oia;
	struct mbuf *m;
	struct sockaddr_ax oldaddr;
	int error, hostIsNew;
	u_long i;

	/*
	 * Find address for this interface, if it exists.
	 */
	if (ifp)
		for (ia = ax_ifaddr; ia; ia = ia->aa_next)
			if (ia->aa_ifp == ifp)
				break;

	switch (cmd) {

	case SIOCAIFADDR:
	case SIOCDIFADDR:
		if (ifra->ifra_addr.sax_family == AF_AX25)
		    for (oia = ia; ia; ia = ia->aa_next) {
				if (ia->aa_ifp == ifp  &&
					ax_addreq(&ia->aa_addr.sax_addr,
						&ifra->ifra_addr.sax_addr) > 1)
					break;
			}
		if (cmd == SIOCDIFADDR && ia == 0)
			return (EADDRNOTAVAIL);
		/* FALLTHROUGH */
	case SIOCSIFADDR:
	case SIOCSIFDSTADDR:
		if ((so->so_state & SS_PRIV) == 0)
			return (EPERM);

		if (ifp == 0)
			panic("ax_control");
		if (ia == NULL) {
			m = m_getclr(M_WAIT, MT_IFADDR);
			if (m == NULL)
				return (ENOBUFS);
			if (ia = ax_ifaddr) {
				for ( ; ia->aa_next; ia = ia->aa_next)
					;
				ia->aa_next = mtod(m, struct ax_ifaddr *);
			} else
				ax_ifaddr = mtod(m, struct ax_ifaddr *);

			ia = mtod(m, struct ax_ifaddr *);
			if (ifa = ifp->if_addrlist) {
				for ( ; ifa->ifa_next; ifa = ifa->ifa_next)
					;
				ifa->ifa_next = (struct ifaddr *) ia;
			} else
				ifp->if_addrlist = (struct ifaddr *) ia;

			ia->aa_ifa.ifa_addr = (struct sockaddr *)&ia->aa_addr;
			ia->aa_ifa.ifa_dstaddr = (struct sockaddr *)&ia->aa_dstaddr;

			if (ifp->if_flags & IFF_BROADCAST) {
				ia->aa_broadaddr.sax_len = sizeof(ia->aa_addr);
				ia->aa_broadaddr.sax_family = AF_AX25;
			}
			ia->aa_ifp = ifp;

			if ((ifp->if_flags & IFF_LOOPBACK) == 0)
				ax_interfaces++;
		}
		break;

	case SIOCSIFBRDADDR:
		if ((so->so_state & SS_PRIV) == 0)
			return (EPERM);
		/* FALLTHROUGH */

	case SIOCGIFADDR:
	case SIOCGIFDSTADDR:
	case SIOCGIFBRDADDR:
		if (ia == NULL)
			return (EADDRNOTAVAIL);
		break;

	default:
		return (EOPNOTSUPP);
		break;
	}
	switch (cmd) {

	case SIOCGIFADDR:
		*((struct sockaddr_ax *)&ifr->ifr_addr) = ia->aa_addr;
		break;

	case SIOCGIFBRDADDR:
		if ((ifp->if_flags & IFF_BROADCAST) == 0)
			return (EINVAL);
		*((struct sockaddr_ax *)&ifr->ifr_dstaddr) = ia->aa_broadaddr;
		break;

	case SIOCGIFDSTADDR:
		if ((ifp->if_flags & IFF_POINTOPOINT) == 0)
			return (EINVAL);
		*((struct sockaddr_ax *)&ifr->ifr_dstaddr) = ia->aa_dstaddr;
		break;

	case SIOCSIFDSTADDR:
		if ((ifp->if_flags & IFF_POINTOPOINT) == 0)
			return (EINVAL);

		oldaddr = ia->aa_dstaddr;
		ia->aa_dstaddr = *(struct sockaddr_ax *)&ifr->ifr_dstaddr;
		if (ifp->if_ioctl &&
				(error = (*ifp->if_ioctl)(ifp, SIOCSIFDSTADDR, ia))) {
			ia->aa_dstaddr = oldaddr;
			return (error);
		}
		if (ia->aa_flags & IFA_ROUTE) {
			ia->aa_ifa.ifa_dstaddr = (struct sockaddr *)&oldaddr;
			rtinit(&(ia->aa_ifa), (int)RTM_DELETE, RTF_HOST);
			ia->aa_ifa.ifa_dstaddr = (struct sockaddr *)&ia->aa_dstaddr;
			rtinit(&(ia->aa_ifa), (int)RTM_ADD, RTF_HOST|RTF_UP);
		}
		break;

	case SIOCSIFBRDADDR:
		if ((ifp->if_flags & IFF_BROADCAST) == 0)
			return (EINVAL);
		ia->aa_broadaddr = *(struct sockaddr_ax *)&ifr->ifr_broadaddr;
		break;

	case SIOCSIFADDR:
		return (ax_ifinit(ifp, ia, (struct sockaddr_ax *) &ifr->ifr_addr, 1));

	case SIOCAIFADDR:
		hostIsNew = 1;
		error = 0;
		if (ia->aa_addr.sax_family == AF_AX25) {
			if (ifra->ifra_addr.sax_len == 0) {
				ifra->ifra_addr = ia->aa_addr;
				hostIsNew = 0;
			} else if (ax_addreq(&ifra->ifra_addr.sax_addr,
					&ia->aa_addr.sax_addr) > 1)
				hostIsNew = 0;
		}
		if ((ifp->if_flags & IFF_POINTOPOINT) &&
				(ifra->ifra_dstaddr.sax_family == AF_AX25)) {
			ax_ifscrub(ifp, ia);
			ia->aa_dstaddr = ifra->ifra_dstaddr;
		}
		if (ifra->ifra_addr.sax_family == AF_AX25 && hostIsNew)
			error = ax_ifinit(ifp, ia, &ifra->ifra_addr, 0);
		if ((ifp->if_flags & IFF_BROADCAST) &&
				(ifra->ifra_broadaddr.sax_family == AF_AX25))
			ia->aa_broadaddr = ifra->ifra_broadaddr;
		return (error);

	case SIOCDIFADDR:
		ax_ifscrub(ifp, ia);
		if ((ifa = ifp->if_addrlist) == (struct ifaddr *)ia)
			ifp->if_addrlist = ifa->ifa_next;
		else {
			while (ifa->ifa_next && (ifa->ifa_next != (struct ifaddr *)ia))
			    ifa = ifa->ifa_next;
			if (ifa->ifa_next)
				ifa->ifa_next = ((struct ifaddr *)ia)->ifa_next;
			else
				printf("Couldn't unlink axifaddr from ifp\n");
		}
		oia = ia;
		if (oia == (ia = ax_ifaddr))
			ax_ifaddr = ia->aa_next;
		else {
			while (ia->aa_next && (ia->aa_next != oia))
				ia = ia->aa_next;
			if (ia->aa_next)
				ia->aa_next = oia->aa_next;
			else
				printf("Didn't unlink axifadr from list\n");
		}
		(void) m_free(dtom(oia));
		break;

	default:
		if (ifp == 0 || ifp->if_ioctl == 0)
			return (EOPNOTSUPP);
		return ((*ifp->if_ioctl)(ifp, cmd, data));
	}
	return (0);
}

/*
 * Delete any existing route for an interface.
 */
ax_ifscrub(ifp, ia)
	register struct ifnet *ifp;
	register struct ax_ifaddr *ia;
{

	if ((ia->aa_flags & IFA_ROUTE) == 0)
		return;
	if (ifp->if_flags & (IFF_LOOPBACK|IFF_POINTOPOINT))
		rtinit(&(ia->aa_ifa), (int)RTM_DELETE, RTF_HOST);
	else
		rtinit(&(ia->aa_ifa), (int)RTM_DELETE, 0);
	ia->aa_flags &= ~IFA_ROUTE;
}

/*
 * Initialize an interface's AX25 address
 * and routing table entry.
 */
ax_ifinit(ifp, ia, sax, scrub)
	register struct ifnet *ifp;
	register struct ax_ifaddr *ia;
	struct sockaddr_ax *sax;
{
	struct sockaddr_ax oldaddr;
	int s = splimp(), error = 0, flags = RTF_UP;

	oldaddr = ia->aa_addr;
	ia->aa_addr = *sax;
	/*
	 * Give the interface a chance to initialize
	 * if this is its first address,
	 * and to validate the address if necessary.
	 */
	if (ifp->if_ioctl && (error = (*ifp->if_ioctl)(ifp, SIOCSIFADDR, ia))) {
		splx(s);
		ia->aa_addr = oldaddr;
		return (error);
	}
	splx(s);
	if (scrub) {
		ia->aa_ifa.ifa_addr = (struct sockaddr *)&oldaddr;
		ax_ifscrub(ifp, ia);
		ia->aa_ifa.ifa_addr = (struct sockaddr *)&ia->aa_addr;
	}
	/*
	 * Add route for the network.
	 */
	if (ifp->if_flags & IFF_BROADCAST) {
		ia->aa_broadaddr.sax_addr = ax_broadcastaddr;
		ia->aa_netbroadcast = ax_broadcastaddr;
	} else if (ifp->if_flags & IFF_LOOPBACK) {
		ia->aa_ifa.ifa_dstaddr = ia->aa_ifa.ifa_addr;
		flags |= RTF_HOST;
	} else if (ifp->if_flags & IFF_POINTOPOINT) {
		if (ia->aa_dstaddr.sax_family != AF_AX25)
			return (0);
		flags |= RTF_HOST;
	}
#ifdef notyet
	if ((error = rtinit(&(ia->aa_ifa), (int)RTM_ADD, flags)) == 0)
		ia->aa_flags |= IFA_ROUTE;
#endif
	return (error);
}

