
; N81 software uart, rx side
; IC2 detects startbit, OC4 times databits
;
; input pin RTS2 = PGP1 = IC2
; no hardware changes needed.

	include regs.inc

IC2_V     equ 84h
OC4_V     equ 8Eh

IC2F      equ 02h ; TFLG1 and TMSK1
OC4F      equ 40h

IC2edges  equ 0Ch ; TCTL1/2
IC2fedge  equ 08h
OC4mode   equ 30h

	;------------------------------------------------------------
	section .bankf
	align 1

bittime     ds.w 1
bittime1_25 ds.w 1
shifter     ds.b 1
            ds.b 1      ; align 1

	section .bss
	align 1

remove      ds.w 1      ; insert is inside .inthand
bufmask equ 0FEh        ; buffer in absolute 0FE00 ... 0FFFF

BUMP MACRO ptr
	incw ptr
	bset ptr, # bufmask
	ENDM

	;------------------------------------------------------------
	section .inthand
	align 1

IC2_int_startbit

	pshm d,k
	ldd  # 80FFh            ; 80 and FF
	tbek
	staa shifter            ; primed

	ldd  bittime1_25        ; 1.25 bits onward lands near the middle of 1st bit
	addd TIC2               ; from the falling edge of startbit
	std  TOC4               ; to middle of data bit 0 (not handling noise)

	bclr TMSK1, # IC2F      ; disable IC2
	bclr TFLG1, # IC2F+OC4F ; ack IC2 and OC4
	bset TMSK1, # OC4F      ; enable OC4

	pulm d,k
	rti

OC4_int_databits

	pshm d,k
	ldab # FF
	tbek

	ldab PORTGP             ; sample rx bit XXX max lag 0.75 bittime
	rorb                    ; put new bit PGP1 = RTSB = RXD3 to ...
	rorb                    ; ... carry
	ror  shifter            ; ... to shifter
	bcs  did_byte           ; if have all of them now

	ldd  bittime            ; near the middle of next bit
	addd TOC4
	std  TOC4
	bclr TFLG1, # OC4F

	pulm d,k
	rti

did_byte

	; last databit has not yet ended, but the next low-going edge (startbit)
	; can happen only after the stop bit. XXX again, noise not handled XXX

	bclr TMSK1, # OC4F      ; disable OC4
	bclr TFLG1, # IC2F+OC4F ; ack IC2 and OC4
	bset TMSK1, # IC2F      ; enable IC2

	ldaa shifter
	clrb
	tbek                    ; EXT RAM

insert = * + 2              ; STAA = 177A address
	staa 0                  ; patched at init
	BUMP insert

	pulm d,k
	rti

	;------------------------------------------------------------
	section .text
	align 1

	global _bbuart_getc

_bbuart_getc

	ldx  remove
	cpx  insert
	beq  L0                 ; if empty
	ldab 0,x
	clra
	BUMP remove
	rts
L0
	ldd  # -1               ; EOF
	rts


	global _bbuart_init

_bbuart_init                ; void _bbuart_init(word baud)

	jsr  soft_uart_bittime

	pshm k,ccr
	orp  # 000E0h           ; IPL 7

	ldd  # IC2_int_startbit
	std  IC2_V
	ldd  # OC4_int_databits
	std  OC4_V

	clrd
	oraa # bufmask
	std  insert
	std  remove

	ldab # FF
	tbek                    ; switch EXT

	ste  bittime            ; one bit
	lsre                    ; half
	lsre                    ; quarter
	adde bittime            ; one and a quarter
	ste  bittime1_25        ; here.

	bclr DDRGP, # IC2F      ; input pin PGP1

	bclr TCTL1, # OC4mode   ; 00 = timer disconnected from output logic

	bclr TCTL2, # IC2edges  ; 00 = capture disabled
	bset TCTL2, # IC2fedge  ; 10 = low-going edge interrupts

	bclr TMSK1, # OC4F      ; disable OC4 interrupt
	bclr TFLG1, # IC2F+OC4F ; forget stale capture
	bset TMSK1, # IC2F      ; enable IC2 interrupt

	pulm k,ccr              ; restore IPL
	rts

; input: baud rate in E
; output: bit time as TCNT ticks in E (yes, E)

soft_uart_bittime
	xgex                    ; baud in IX
	lde  # 1048576 / 65536  ; 16.777 MHz / 16 / baud
	ldd  # 1048576 % 65536
	ediv
	xgex                    ; time constant in E
	rts

