
#include "standard.h"
#include "ieee_fp.h"

/* 32-bit IEEE 1-bit sign, 8-bit excess-127 exponent and 23-bit mantissa */

long
ieee_float_to_long(ieee_float f, int scale)
{
	int   exp = ld_word_intel(f + 2) >> 7;     /* 9 MSbits */
	dword num = ld_dword_intel(f) & 0x7FFFFF;  /* 23-bit mantissa */

	num += 0x00800000; /* together at last */
	exp &= 0xFF;       /* 8-bit exponent */
	exp -= 23;

	exp += scale;     /* scale n binades up or down */

	exp -= 127;
	if (exp > 0)
		ASLx(&num, exp);
	if (exp < 0)
		LSRx(&num, -exp);

	if (f[3] & 0x80)
		lnegate(&num);

	return (num);
}

/* 64-bit IEEE 1-bit sign, 11-bit excess-1023 exponent and 52-bit mantissa */

long
ieee_double_to_long(ieee_double d, int scale)
{
	int   exp = (ld_word_intel(d + 6) >> 4) & 0x7FF; /* 11 almost MSbits */
	dword num = ld_dword_intel(d + 3) & 0xFFFFFFF;   /* 28 bits */

	ASLx(&num, 3);       /* fish for 3 more */
	num += d[2] >> 5;
	num += 0x80000000;
	exp -= 31;           /* 31 fractional bits */

	exp += scale;

	exp -= 1023;
	if (exp > 0)
		ASLx(&num, exp);
	if (exp < 0)
		LSRx(&num, -exp);

	if (d[7] & 0x80)
		lnegate(&num);

	return (num);
}

