
#include "standard.h"
#include "wx.h"


#define DATE        11, 0x023B /* mMhHwdDmMyY */
#define TEMPERATURE  4, 0x0373 /* .01 .1 1 10, offset 30 celsius */
#define DEWPOINT     4, 0x03CE /* .01 .1 1 10, celsius */
#define HUMIDITY     2, 0x0419 /* 1 10, percent */
#define RAIN_24H     6, 0x0497 /* .01 .1 1 10 100 1000, mm */
#define RAIN_1H      6, 0x04B4 /* .01 .1 1 10 100 1000, mm */
#define RAIN_TOTAL   6, 0x04D2 /* .01 .1 1 10 100 1000, mm */
#define WIND         4, 0x0529 /* 3 binary nybbles, 0.1 m/s AND direction */
#define PRESSURE     5, 0x05D8 /* .1 1 10 100 1000, hPa */

/* indexed by last nybble in WIND */

static word
wind_degrees[16] = {
	0, 22, 45, 68, 90, 112, 135, 158, 180,
	202, 225, 248, 270, 292, 315, 338
};

/* mph from 0.1 m/s, factor 0.22369363 */

static word
wind_mph(byte *wdat)
{
	return (muldiv(ld_word_intel(wdat) & 0xFFF, 625, 2794));
};

static word
wind_dir(byte *wdat)
{
	return (wind_degrees[wdat[1] >> 4]);
}

#define putc SCI_putc

static void
reset()
{
	putc(0x06);
}

static void
address_nybble(byte n)
{
	putc(0x82 | ((n & 15) << 2));
}

static void
cmd_read(byte n)
{
	putc(0xC2 | ((n & 15) << 2));
}

static void
set_address(word address)
{
	address_nybble(0);               /* always zero */
	address_nybble(address >> 8);
	address_nybble(address >> 4);
	address_nybble(address >> 0);
}

static bool
checksum(byte *p, word nbytes)
{
	byte sum = 0;

	do {
		sum += *p++;
	} while (--nbytes);

	return (*p == sum);
}

static word
unbcd(byte *dcb, int nnyble)
{
	int odd = nnyble & 1;
	word bin = 0;

	dcb += nnyble / 2; /* over last if odd, one past last if even */

	do {
		bin *= 10;
		bin += (odd ? (*dcb & 15) : (*--dcb >> 4));
		odd ^= 1;
	} while (--nnyble);

	return (bin);
}

static stamp mark;
static byte buffer[32], *bufptr = buffer;
static word bytes_needed;

static void
post_read(word address, word nnybble)
{
	bytes_needed = (nnybble + 1) / 2; 

	reset();                 /* returns 0x02 */
	set_address(address);    /* returns 0x0a 0x1b 0x2x 0x3d */
	cmd_read(bytes_needed);  /* returns 0x3z */
}

static void
handle(, byte *buf)
{
	temperature_celsius    = ;
	temperature_fahrenheit = ;
	have_thermometer = TRUE;

	relative_humidity = ;

	have_hygrometer = TRUE;

	have_barometer = TRUE;
}

void
lacrosse_wx_input(byte c)
{
	if (bufptr < __ENDOF(buffer))
		*bufptr++ = c;
}

void
lacrosse_wx_check()
{
	if (time_past(mark)) {
		future_msec(&mark, 1000);

		/*
		 * last n + 1 bytes should be the response plus checksum.
		 * first 6 bytes is the "echo" of previous command.
		 */

		if (bufptr < __ENDOF(buffer)
		 && (bufptr -= bytes_needed + 1) >= buffer + 6)
			if (checksum(bufptr, bytes_needed))
				handle(, bufptr);

		/* next */

		bufptr = buffer;
		post_read(->addr, ->nnybble);
	}
}

void
lacross_wx_init()
{
	/* 2400N8 */

	(2400);
}

